@@ -141,6 +141,11 @@ class parseAddress(unittest.TestCase):
         """'1111:2222:3333:4444:5555:6666:7777:8888:9999' should raise an exception"""
         self.assertRaises(ValueError, IPy.parseAddress, '1111:2222:3333:4444:5555:6666:7777:8888:9999')
 
+    def testBogusValues(self):
+        """Text values should raise an exception"""
+        self.assertRaises(ValueError, IPy.parseAddress, 'xx')
+        self.assertRaises(ValueError, IPy.parseAddress, 'foobar')
+
 class _intToIP(unittest.TestCase):
     v4values = [(0x7f000001, '127.0.0.1'),
                 (0x0, '0.0.0.0'),
@@ -445,6 +450,13 @@ class PythonObjectBehaviour(unittest.TestCase):
         for x in ip:
             self.assertTrue(x in ip)
 
+    def testContainsVersionSeparation(self):
+        """__contains__() should return false if versions mismatch"""
+        four = IPy.IP('192.168.0.0/16')
+        six = IPy.IP('::c0a8:0/112')
+        self.assertFalse(four in six)
+        self.assertFalse(six in four)
+
     def testActingAsArray(self):
         """An IP-object should handle indices."""
         ip = IPy.IP('127.0.0.0/24')
@@ -678,6 +690,14 @@ class IPobject(unittest.TestCase):
             result = IPy.IP(question).strHex(question).lower()
             self.assertEqual(answer, result, (question, result, answer))
 
+    def testV46map(self):
+        four    = IPy.IP('192.168.1.1')
+        six     = IPy.IP('::ffff:192.168.1.1')
+        invalid = IPy.IP('2001::ffff:192.168.1.1')
+        self.assertEqual(four.v46map(), six)
+        self.assertEqual(four, six.v46map())
+        self.assertRaises(ValueError, invalid.v46map)
+
 # TODO
 #eval(repr(IPy))
 # differences between IP and IPint
@@ -715,7 +735,12 @@ class NetIPChecks(unittest.TestCase):
 
         ip = IPy.IP('133.45.0/24')
         ip2 = IPy.IP('133.45.1/24')
+        ip3 = IPy.IP('133.45.2/24')
         self.assertEqual((ip + ip2).prefixlen(),23)
+        # Non-adjacent ranges
+        self.assertRaises(ValueError, IPy.IP.__add__, ip, ip3)
+        # Resulting invalid prefix
+        self.assertRaises(ValueError, IPy.IP.__add__, ip2, ip3)
 
         ip2 = IPy.IP('133.44.255.255');
         #$T->ok_eqnum ($ip->bincomp('gt',$ip2),1,$ip->error());
@@ -756,6 +781,7 @@ class NetIPChecks(unittest.TestCase):
         ip = IPy.IP('::e000:0/112')
         ip2 = IPy.IP('::e001:0/112')
         self.assertEqual(ip.__add__(ip2).prefixlen(),111)
+        self.assertEqual(ip.__add__(ip2).version(),6)
 
         ip2 = IPy.IP('::dfff:ffff')
         #$T->ok_eqnum ($ip->bincomp('gt',$ip2),1,$ip->error());
@@ -796,6 +822,32 @@ def timeout(func, args=(), kwargs={}, timeout_duration=1, default=None):
     else:
         return it.result
 
+class IPSetChecks(unittest.TestCase):
+    def setUp(self):
+        #array
+        self.a = [IPy.IP("192.168." + str(i) + ".0/24") for i in range(256)]
+        #range
+        self.r = IPy.IP('192.168.0.0/16')
+        #testing set
+        self.t = IPy.IPSet(self.a)
+        #control set
+        self.c = IPy.IPSet(self.a)
+        #Could otherwise look like 192.168.128.0/17
+        self.sixRange = IPy.IP('::c0a8:8000/113')
+
+    def testVersionSeparation(self):
+        #Don't remove a matching IPv6 subnet from an IPv4 list
+        self.assertRaises(KeyError, self.t.remove, self.sixRange)
+        self.t.add(self.sixRange)
+        self.assertNotEqual(self.t, self.c)
+        self.t.remove(self.sixRange)
+        self.t.discard(self.sixRange)
+        self.assertEqual(self.t, self.c)
+
+    def testContains(self):
+        self.assertTrue(IPy.IP('192.168.15.32/28') in self.t)
+        self.assertFalse(IPy.IP('192.169.15.32/28') in self.t)
+
 class RegressionTest(unittest.TestCase):
     def testNulNetmask(self):
         ip = timeout(IPy.IP, ["0.0.0.0/0.0.0.0"], timeout_duration=0.250, default=None)
